@@ -22,6 +22,7 @@
 #include <string.h>
 #include <eel/eel.h>
 
+#include "seahorse-gpgmex.h"
 #include "seahorse-op.h"
 #include "seahorse-util.h"
 #include "seahorse-context.h"
@@ -40,28 +41,32 @@ set_gpgme_opts (SeahorseKeySource *sksrc)
 /* helper function for importing @data. @data will be released.
  * returns number of keys imported or -1. */
 static gint
-import_data (SeahorseKeySource *sksrc, gpgme_data_t data, gpgme_error_t *err)
+import_data (SeahorseKeySource *sksrc, gpgme_data_t data, 
+             GError **err)
 {
+    SeahorseOperation *operation;
+    GList *keylist;
 	gint keys = 0;
-    gpgme_ctx_t new_ctx;
-
-    new_ctx = seahorse_key_source_new_context (sksrc);	
-    g_return_val_if_fail (new_ctx != NULL, -1);
     
-	*err = gpgme_op_import (new_ctx, data);
-	if (GPG_IS_OK (*err)) {
-        gpgme_import_result_t result = gpgme_op_import_result (new_ctx);
-		keys = result->considered;
-	}
-	gpgme_data_release (data);
-    gpgme_release (new_ctx);
-	
-	if (GPG_IS_OK (*err)) {
-        seahorse_key_source_refresh (sksrc, FALSE);
-		return keys;
-	}
-	else
-		g_return_val_if_reached (-1);
+    g_return_val_if_fail (!err || !err[0], -1);
+
+    operation = seahorse_key_source_import (sksrc, data);
+    g_return_val_if_fail (operation != NULL, -1);
+    
+    seahorse_operation_wait (operation);
+    
+    if (seahorse_operation_is_successful (operation)) {
+        keylist = (GList*)seahorse_operation_get_result (operation);
+        keys = g_list_length (keylist);
+    } else {
+        seahorse_operation_steal_error (operation, err);
+        keys = -1;
+    }
+    
+    g_object_unref (operation);
+    gpgme_data_release (data);
+
+    return keys;    
 }
 
 /**
@@ -75,17 +80,18 @@ import_data (SeahorseKeySource *sksrc, gpgme_data_t data, gpgme_error_t *err)
  * Returns: Number of keys imported or -1 if import fails
  **/
 gint
-seahorse_op_import_file (SeahorseKeySource *sksrc, const gchar *path, gpgme_error_t *err)
+seahorse_op_import_file (SeahorseKeySource *sksrc, const gchar *path, GError **err)
 {
 	gpgme_data_t data;
-	gpgme_error_t error;
+	gpgme_error_t gerr;
   
-	if (err == NULL)
-		err = &error;
 	/* new data from file */
-    data = seahorse_vfs_data_create (path, FALSE, err);
-    g_return_val_if_fail (data != NULL, -1);
-	
+    data = seahorse_vfs_data_create (path, SEAHORSE_VFS_READ, &gerr);
+    if (!GPG_IS_OK (gerr)) {
+        seahorse_util_gpgme_to_error (gerr, err);
+        return -1;
+    }
+    
 	return import_data (sksrc, data, err);
 }
 
@@ -100,63 +106,73 @@ seahorse_op_import_file (SeahorseKeySource *sksrc, const gchar *path, gpgme_erro
  * Returns: Number of keys imported or -1 if import fails
  **/
 gint
-seahorse_op_import_text (SeahorseKeySource *sksrc, const gchar *text, gpgme_error_t *err)
+seahorse_op_import_text (SeahorseKeySource *sksrc, const gchar *text, GError **err)
 {
 	gpgme_data_t data;
-	gpgme_error_t error;
+    gpgme_error_t gerr;
 	
-	if (err == NULL)
-		err = &error;
-
     g_return_val_if_fail (text != NULL, 0);
          
 	/* new data from text */
-	*err = gpgme_data_new_from_mem (&data, text, strlen (text), TRUE);
-	g_return_val_if_fail (GPG_IS_OK (*err), -1);
-	
+	gerr = gpgme_data_new_from_mem (&data, text, strlen (text), TRUE);
+    if (!GPG_IS_OK (gerr)) {
+        seahorse_util_gpgme_to_error (gerr, err);
+        g_return_val_if_reached (-1);
+    }	
+    
 	return import_data (sksrc, data, err);
 }
 
-/* helper function for exporting @keys. returns exported data. */
-static gpgme_data_t
-export_data (GList *keys, gboolean force_armor, gpgme_error_t *err)
+/* helper function for exporting @keys. */
+static gboolean
+export_data (GList *keys, gboolean force_armor, gpgme_data_t data,
+             GError **err)
 {
     SeahorseKeySource *sksrc;
+    SeahorseOperation *operation;
+    gboolean ret = TRUE;
     SeahorseKey *skey;
-	gpgme_data_t data = NULL;
-    gpgme_error_t error;
-    gboolean old_armor;
-    GList *l;
-   
-    if (err == NULL)
-        err = &error;
-       
-	*err = gpgme_data_new (&data);
+    GList *next;
+    
+    keys = g_list_copy (keys);
 
-    for (l = keys; l != NULL; l = g_list_next (l)) {
-       
-        g_return_val_if_fail (SEAHORSE_IS_KEY (l->data), data);
-        skey = SEAHORSE_KEY (l->data);
-       
+    /* Sort by key source */
+    keys = seahorse_util_keylist_sort (keys);
+    
+    while (keys) {
+     
+        /* Break off one set (same keysource) */
+        next = seahorse_util_keylist_splice (keys);
+
+        g_return_val_if_fail (SEAHORSE_IS_KEY (keys->data), FALSE);
+        skey = SEAHORSE_KEY (keys->data);
+
+        /* Export from this key source */        
         sksrc = seahorse_key_get_source (skey);
-        g_return_val_if_fail (sksrc != NULL, data);
+        g_return_val_if_fail (sksrc != NULL, FALSE);
+        
+        /* We pass our own data object, to which data is appended */
+        operation = seahorse_key_source_export (sksrc, keys, data);
+        g_return_val_if_fail (operation != NULL, FALSE);
 
-        set_gpgme_opts (sksrc);
+        g_list_free (keys);
+        keys = next;
         
-        if (force_armor) {
-            old_armor = gpgme_get_armor (sksrc->ctx);
-            gpgme_set_armor (sksrc->ctx, TRUE);
-        }
+        seahorse_operation_wait (operation);
+    
+        if (!seahorse_operation_is_successful (operation)) {
+            seahorse_operation_steal_error (operation, err);
+
+            /* Ignore the rest, break loop */
+            g_list_free (keys);
+            keys = NULL;
+            ret = FALSE;
+        }        
         
-        *err = gpgme_op_export (sksrc->ctx, skey->key->subkeys->fpr, 0, data);
-        
-        if (force_armor)
-            gpgme_set_armor (sksrc->ctx, old_armor);
-        
-        g_return_val_if_fail (GPG_IS_OK (*err), data);
-    }
- 
-	return data;
+        g_object_unref (operation);
+    } 
+    
+    return ret;
 }
 
 /**
@@ -168,19 +184,24 @@ export_data (GList *keys, gboolean force_armor, gpgme_error_t *err)
  * Tries to export @recips to the new file @path, saving an errors in @err.
  * @recips will be released upon completion.
  **/
-void
-seahorse_op_export_file (GList *keys, const gchar *path, gpgme_error_t *err)
+gboolean
+seahorse_op_export_file (GList *keys, const gchar *path, GError **err)
 {
 	gpgme_data_t data;
-	gpgme_error_t error;
+	gpgme_error_t gerr;
+    gboolean ret;
 	
-	if (err == NULL)
-		err = &error;
+    /* Open the appropriate file */
+    data = seahorse_vfs_data_create (path, SEAHORSE_VFS_WRITE, &gerr);
+    if (!GPG_IS_OK (gerr)) {
+        seahorse_util_gpgme_to_error (gerr, err);
+        return FALSE;
+    }
+    
 	/* export data */
-	data = export_data (keys, FALSE, err);
-	g_return_if_fail (GPG_IS_OK (*err));
-	/* write to file */
-	*err = seahorse_util_write_data_to_file (path, data);
+	ret = export_data (keys, FALSE, data, err);
+    gpgme_data_release (data);
+    return ret;
 }
 
 /**
@@ -195,19 +216,24 @@ seahorse_op_export_file (GList *keys, const gchar *path, gpgme_error_t *err)
  * Returns: The exported text or NULL if the operation fails
  **/
 gchar*
-seahorse_op_export_text (GList *keys, gpgme_error_t *err)
+seahorse_op_export_text (GList *keys, GError **err)
 {
 	gpgme_data_t data;
-	gpgme_error_t error;
-	
-	if (err == NULL)
-		err = &error;
-	
+	gpgme_error_t gerr;
+        
+    gerr = gpgme_data_new (&data);
+    if (!GPG_IS_OK (gerr)) {
+        seahorse_util_gpgme_to_error (gerr, err);
+        g_return_val_if_reached (NULL);
+    }  
+    
 	/* export data with armor */
-	data = export_data (keys, TRUE, err);
-	g_return_val_if_fail (GPG_IS_OK (*err), NULL);
-	
-	return seahorse_util_write_data_to_text (data);
+	if (export_data (keys, TRUE, data, err)) {
+    	return seahorse_util_write_data_to_text (data);
+    } else {
+        gpgme_data_release (data);
+        return NULL;
+    }
 }
 
 /* common encryption function definition. */
@@ -216,80 +242,82 @@ typedef gpgme_error_t (* EncryptFunc) (gpgme_ctx_t ctx, gpgme_key_t recips[],
 				       gpgme_data_t plain, gpgme_data_t cipher);
 
 /* helper function for encrypting @plain to @recips using @func. @plain and
- * @recips will be released. returns the encrypted data. */
-static gpgme_data_t
+ * @keys will be released.*/
+static void
 encrypt_data_common (SeahorseKeySource *sksrc, GList *keys, gpgme_data_t plain, 
-                     EncryptFunc func, gpgme_error_t *err)
+                     gpgme_data_t cipher, EncryptFunc func, gboolean force_armor, 
+                     gpgme_error_t *err)
 {
     SeahorseKey *skey;
-	gpgme_data_t cipher = NULL;
     gpgme_key_t *recips;
+    gboolean old_armor;
     gchar *id;
 
 	/* if don't already have an error, do encrypt */
 	if (GPG_IS_OK (*err)) {
 
-        set_gpgme_opts (sksrc);
-        
         /* Add the default key if set and necessary */
         if (eel_gconf_get_boolean (ENCRYPTSELF_KEY)) {
             id = eel_gconf_get_string (DEFAULT_KEY);        
             if (id != NULL) {
-                skey = seahorse_key_source_get_key (sksrc, id, TRUE);
+                skey = seahorse_key_source_get_key (sksrc, id);
                 if (skey != NULL)
                     keys = g_list_append (keys, skey);
             }
         }
 
         /* Make keys into the right format */
-        recips = seahorse_util_list_to_keys (keys);
+        recips = seahorse_util_keylist_to_keys (keys);
         
-		*err = gpgme_data_new (&cipher);
-		if (GPG_IS_OK (*err))
-			*err = func (sksrc->ctx, recips, GPGME_ENCRYPT_ALWAYS_TRUST, plain, cipher);
+        set_gpgme_opts (sksrc);
+
+        if (force_armor) {
+            old_armor = gpgme_get_armor (sksrc->ctx);
+            gpgme_set_armor (sksrc->ctx, TRUE);
+        }
+              
+		*err = func (sksrc->ctx, recips, GPGME_ENCRYPT_ALWAYS_TRUST, plain, cipher);
+        
+        if (force_armor)
+            gpgme_set_armor (sksrc->ctx, old_armor);
          
         seahorse_util_free_keys (recips);
 	}
-	/* release plain and recips */
+	/* release plain  */
 	gpgme_data_release (plain);
-	return cipher;
 }
 
 /* common file encryption helper to encrypt @path to @recips using @func.
- * returns the encrypted file's path. */
-static gchar*
-encrypt_file_common (GList *keys, const gchar *path, EncryptFunc func, 
-                     gpgme_error_t *err)
+ * @keys will be released.*/
+static void
+encrypt_file_common (GList *keys, const gchar *path, const gchar *epath,
+                     EncryptFunc func, gpgme_error_t *err)
 {
     SeahorseKeySource *sksrc;
 	gpgme_data_t plain, cipher;
 	gpgme_error_t error;
-	gchar *new_path = NULL;
 	
 	if (err == NULL)
 		err = &error;
 
     /* When S/MIME supported need to make sure all keys from the same source */       
-    g_return_val_if_fail (keys && SEAHORSE_IS_KEY (keys->data), NULL);
+    g_return_if_fail (keys && SEAHORSE_IS_KEY (keys->data));
     sksrc = seahorse_key_get_source (SEAHORSE_KEY (keys->data));
-    g_return_val_if_fail (sksrc != NULL, NULL);
+    g_return_if_fail (sksrc != NULL);
 
-    plain = seahorse_vfs_data_create (path, FALSE, err);
-    g_return_val_if_fail (plain != NULL, NULL);
+    plain = seahorse_vfs_data_create (path, SEAHORSE_VFS_READ, err);
+    g_return_if_fail (plain != NULL);
+    
+    cipher = seahorse_vfs_data_create (epath, SEAHORSE_VFS_WRITE | SEAHORSE_VFS_DELAY, err);
+    if (!cipher) {
+        gpgme_data_release (plain);
+        g_return_if_reached ();
+    }
  
-	cipher = encrypt_data_common (sksrc, keys, plain, func, err);
-	g_return_val_if_fail (GPG_IS_OK (*err), NULL);
-	/* write cipher to file */
-	new_path = seahorse_util_add_suffix (sksrc->ctx, path, SEAHORSE_CRYPT_SUFFIX);
-	*err = seahorse_util_write_data_to_file (new_path, cipher);
-	
-	/* free new_path if error */
-	if (!GPG_IS_OK (*err)) {
-		g_free (new_path);
-		g_return_val_if_reached (NULL);
-	}
-	
-	return new_path;
+	encrypt_data_common (sksrc, keys, plain, cipher, func, FALSE, err);
+	g_return_if_fail (GPG_IS_OK (*err));
+
+    gpgme_data_release (cipher);	
 }
 
 /* common text encryption helper to encrypt @text to @recips using @func.
@@ -301,7 +329,6 @@ encrypt_text_common (GList *keys, const gchar *text, EncryptFunc func,
     SeahorseKeySource *sksrc;
 	gpgme_data_t plain, cipher;
 	gpgme_error_t error;
-	gboolean armor;
 	
 	if (err == NULL)
 		err = &error;
@@ -313,11 +340,13 @@ encrypt_text_common (GList *keys, const gchar *text, EncryptFunc func,
     
 	/* new data form text */
 	*err = gpgme_data_new_from_mem (&plain, text, strlen (text), TRUE);
+    g_return_val_if_fail (GPG_IS_OK (*err), NULL);
+    
+    *err = gpgme_data_new (&cipher);
+    g_return_val_if_fail (GPG_IS_OK (*err), NULL);
+   
 	/* encrypt with armor */
-	armor = gpgme_get_armor (sksrc->ctx);
-	gpgme_set_armor (sksrc->ctx, TRUE);
-	cipher = encrypt_data_common (sksrc, keys, plain, func, err);
-	gpgme_set_armor (sksrc->ctx, armor);
+	encrypt_data_common (sksrc, keys, plain, cipher, func, TRUE, err);
 	g_return_val_if_fail (GPG_IS_OK (*err), NULL);
 	
 	return seahorse_util_write_data_to_text (cipher);
@@ -327,19 +356,16 @@ encrypt_text_common (GList *keys, const gchar *text, EncryptFunc func,
  * seahorse_op_encrypt_file:
  * @keys: List of #SeahorseKey objects to encrypt to
  * @path: Path of file to encrypt
+ * @epath: Path of file to write encrypted data
  * @err: Optional error value
  *
  * Tries to encrypt the file @path to @recips, saving any errors in @err.
- * The encrypted file's encoding and suffix depends on the ASCII Armor setting of @sctx.
- * If ASCII Armor is enabled, the suffix will be '.asc'. Otherwise it will be '.gpg'.
- * @recips will be released upon completion.
- *
- * Returns: The path of the encrypted file or NULL if encryption fails
  **/
-gchar*
-seahorse_op_encrypt_file (GList *keys, const gchar *path, gpgme_error_t *err)
+void
+seahorse_op_encrypt_file (GList *keys, const gchar *path, const gchar *epath,
+                          gpgme_error_t *err)
 {
-	return encrypt_file_common (keys, path, gpgme_op_encrypt, err);
+	return encrypt_file_common (keys, path, epath, gpgme_op_encrypt, err);
 }
 
 /**
@@ -372,72 +398,59 @@ set_signer (SeahorseKeyPair *signer)
     gpgme_signers_add (sksrc->ctx, signer->secret);
 }
 
-/* helper function for signing @plain with @mode. @plain will be released.
- * returns the signed data. */
-static gpgme_data_t
-sign_data (SeahorseKeySource *sksrc, gpgme_data_t plain, gpgme_sig_mode_t mode, 
-           gpgme_error_t *err)
+/* helper function for signing @plain with @mode. @plain will be released. */
+static void
+sign_data (SeahorseKeySource *sksrc, gpgme_data_t plain, gpgme_data_t sig,
+           gpgme_sig_mode_t mode, gpgme_error_t *err)
 {
-	gpgme_data_t sig = NULL;
-
     set_gpgme_opts (sksrc);
 
-	*err = gpgme_data_new (&sig);
-	if (GPG_IS_OK (*err))
-		*err = gpgme_op_sign (sksrc->ctx, plain, sig, mode);
-	
+	*err = gpgme_op_sign (sksrc->ctx, plain, sig, mode);
+    	
 	gpgme_data_release (plain);
-	return sig;
 }
 
 /**
  * seahorse_op_sign_file:
  * @signer: Key pair to sign with 
  * @path: Path of file to sign
+ * @spath: Where to write the signature
  * @err: Optional error value
  *
  * Tries to create a detached signature file for the file @path, saving any errors
- * in @err. If ASCII Armor is enabled, the signature file will have a suffix of '.asc',
- * otherwise the suffix will be '.sig'. Signing will be done by the default key
- * of @sctx.
- *
- * Returns: The path of the signature file or NULL if the operation fails
+ * in @err. 
  **/
-gchar*
+void
 seahorse_op_sign_file (SeahorseKeyPair *signer, const gchar *path, 
-                       gpgme_error_t *err)
+                       const gchar *spath, gpgme_error_t *err)
 {
     SeahorseKeySource *sksrc;
 	gpgme_data_t plain, sig;
 	gpgme_error_t error;
-	gchar *new_path = NULL;
 	
 	if (err == NULL)
 		err = &error;
 
     sksrc = seahorse_key_get_source (SEAHORSE_KEY (signer));
-    g_return_val_if_fail (sksrc != NULL, NULL);
+    g_return_if_fail (sksrc != NULL);
     
 	/* new data from file */
-    plain = seahorse_vfs_data_create (path, FALSE, err);
-    g_return_val_if_fail (plain != NULL, NULL);
+    plain = seahorse_vfs_data_create (path, SEAHORSE_VFS_READ, err);
+    g_return_if_fail (plain != NULL);
+    
+    sig = seahorse_vfs_data_create (spath, SEAHORSE_VFS_WRITE | SEAHORSE_VFS_DELAY, err);
+    if (!sig) {
+        gpgme_data_release (plain);
+        g_return_if_reached ();
+    }
   
     set_signer (signer);
     
 	/* get detached signature */
-	sig = sign_data (sksrc, plain, GPGME_SIG_MODE_DETACH, err);
-	g_return_val_if_fail (GPG_IS_OK (*err), NULL);
-	/* write sig to file */
-	new_path = seahorse_util_add_suffix (sksrc->ctx, path, SEAHORSE_SIG_SUFFIX);
-	*err = seahorse_util_write_data_to_file (new_path, sig);
-	
-	/* free new_path if error */
-	if (!GPG_IS_OK (*err)) {
-		g_free (new_path);
-		g_return_val_if_reached (NULL);
-	}
-	
-	return new_path;
+	sign_data (sksrc, plain, sig, GPGME_SIG_MODE_DETACH, err);
+	g_return_if_fail (GPG_IS_OK (*err));
+  
+    gpgme_data_release (sig);
 }
 
 /**
@@ -470,8 +483,12 @@ seahorse_op_sign_text (SeahorseKeyPair *signer, const gchar *text,
 	/* new data from text */
 	*err = gpgme_data_new_from_mem (&plain, text, strlen (text), TRUE);
 	g_return_val_if_fail (GPG_IS_OK (*err), NULL);
+    
+    *err = gpgme_data_new (&sig);
+    g_return_val_if_fail (GPG_IS_OK (*err), NULL);    
+    
 	/* clear sign data already ignores ASCII Armor */
-	sig = sign_data (sksrc, plain, GPGME_SIG_MODE_CLEAR, err);
+	sign_data (sksrc, plain, sig, GPGME_SIG_MODE_CLEAR, err);
 	g_return_val_if_fail (GPG_IS_OK (*err), NULL);
 	
 	return seahorse_util_write_data_to_text (sig);
@@ -481,21 +498,20 @@ seahorse_op_sign_text (SeahorseKeyPair *signer, const gchar *text,
  * seahorse_op_encrypt_sign_file:
  * @keys: List of #SeahorseKey objects to encrypt to
  * @path: Path of file to encrypt and sign
+ * @epath: Where to write encrypted data
  * @recips: Keys to encrypt with
  * @err: Optional error value
  *
  * Tries to encrypt and sign the file @path to @recips, saving any errors in @err.
  * Signing will be done with the default key of @sctx. @recips will be released
  * upon completion.
- *
- * Returns: The path of the encrypted and signed file, or NULL if the operation fails
  **/
-gchar*
+void
 seahorse_op_encrypt_sign_file (GList *keys, SeahorseKeyPair *signer, 
-                               const gchar *path, gpgme_error_t *err)
+                               const gchar *path, const gchar *epath, gpgme_error_t *err)
 {
     set_signer (signer);
-	return encrypt_file_common (keys, path, gpgme_op_encrypt_sign, err);
+	encrypt_file_common (keys, path, epath, gpgme_op_encrypt_sign, err);
 }
 
 /**
@@ -540,12 +556,10 @@ seahorse_op_verify_file (SeahorseKeySource *sksrc, const gchar *path, const gcha
 	if (err == NULL)
 		err = &error;
 	/* new data from sig file */
-    sig = seahorse_vfs_data_create (path, FALSE, err);
+    sig = seahorse_vfs_data_create (path, SEAHORSE_VFS_READ, err);
     g_return_if_fail (plain != NULL);
 
-    plain = seahorse_vfs_data_create (original, FALSE, err);
-  
-	/* release sig data if error */
+    plain = seahorse_vfs_data_create (original, SEAHORSE_VFS_READ, err);
 	if (!plain) {
 		gpgme_data_release (sig);
 		g_return_if_reached ();
@@ -605,68 +619,57 @@ seahorse_op_verify_text (SeahorseKeySource *sksrc, const gchar *text,
 	return seahorse_util_write_data_to_text (plain);
 }
 
-/* helper function to decrypt and verify @cipher. @cipher will be released.
- * returns the verified data. */
-static gpgme_data_t
+/* helper function to decrypt and verify @cipher. @cipher will be released. */
+static void
 decrypt_verify_data (SeahorseKeySource *sksrc, gpgme_data_t cipher,
-                     gpgme_verify_result_t *status, gpgme_error_t *err)
+                     gpgme_data_t plain, gpgme_verify_result_t *status, 
+                     gpgme_error_t *err)
 {
-	gpgme_data_t plain;
-
     set_gpgme_opts (sksrc);
 	
-	*err = gpgme_data_new (&plain);
-	if (GPG_IS_OK (*err))
-		*err = gpgme_op_decrypt_verify (sksrc->ctx, cipher, plain);
-    
+	*err = gpgme_op_decrypt_verify (sksrc->ctx, cipher, plain);
+        
     if (status)
     	*status = gpgme_op_verify_result (sksrc->ctx);
      
 	gpgme_data_release (cipher);
-	return plain;
 }
 
 /**
  * seahorse_op_decrypt_verify_file:
  * @sksrc: #SeahorseKeySource to verify against
  * @path: Path of file to decrypt and verify
+ * @path: Where to write the decrypted data
  * @status: Will contain the status of any verified signatures
  * @err: Optional error value
  *
  * Tries to decrypt and verify the file @path, saving any errors in @err. The
- * status of any verified signatures will be saved in @status. A new file
- * will be created to contain the decrypted data. Its path will be the same as
- * @path without the suffix.
- *
- * Returns: The path of the decrypted file or NULL if the operation fails
+ * status of any verified signatures will be saved in @status. 
  **/
-gchar*
-seahorse_op_decrypt_verify_file (SeahorseKeySource *sksrc, const gchar *path,
-                                 gpgme_verify_result_t *status, gpgme_error_t *err)
+void
+seahorse_op_decrypt_verify_file (SeahorseKeySource *sksrc, const gchar *path, 
+                                 const gchar *dpath, gpgme_verify_result_t *status, 
+                                 gpgme_error_t *err)
 {
 	gpgme_data_t cipher, plain;
 	gpgme_error_t error;
-	gchar *new_path = NULL;
 	
 	if (err == NULL)
 		err = &error;
 	/* new data from file */
-    cipher = seahorse_vfs_data_create (path, FALSE, err);
-    g_return_val_if_fail (plain != NULL, NULL);
+    cipher = seahorse_vfs_data_create (path, SEAHORSE_VFS_READ, err);
+    g_return_if_fail (cipher != NULL);
+    
+    plain = seahorse_vfs_data_create (dpath, SEAHORSE_VFS_WRITE | SEAHORSE_VFS_DELAY, err);
+    if (!plain) {
+        gpgme_data_release (cipher);
+        g_return_if_reached ();
+    }
 
 	/* verify data */
-	plain = decrypt_verify_data (sksrc, cipher, status, err);
-	g_return_val_if_fail (GPG_IS_OK (*err), NULL);
-	/* write data to new file */
-	new_path = seahorse_util_remove_suffix (path);
-	*err = seahorse_util_write_data_to_file (new_path, plain);
-	/* free path if there is an error */
-	if (!GPG_IS_OK (*err)) {
-		g_free (new_path);
-		g_return_val_if_reached (NULL);
-	}
-	
-	return new_path;
+	decrypt_verify_data (sksrc, cipher, plain, status, err);
+
+    gpgme_data_release (plain);
 }
 
 /**
@@ -695,10 +698,14 @@ seahorse_op_decrypt_verify_text (SeahorseKeySource *sksrc, const gchar *text,
 	/* new data from text */
 	*err = gpgme_data_new_from_mem (&cipher, text, strlen (text), TRUE);
 	g_return_val_if_fail (GPG_IS_OK (*err), NULL);
+    
+    *err = gpgme_data_new (&plain);
+    g_return_val_if_fail (GPG_IS_OK (*err), NULL);
+    
 	/* get decrypted data with armor */
 	armor = gpgme_get_armor (sksrc->ctx);
 	gpgme_set_armor (sksrc->ctx, TRUE);
-	plain = decrypt_verify_data (sksrc, cipher, status, err);
+	decrypt_verify_data (sksrc, cipher, plain, status, err);
 	gpgme_set_armor (sksrc->ctx, armor);
 	g_return_val_if_fail (GPG_IS_OK (*err), NULL);
 	/* return text of decrypted data */
