@@ -26,6 +26,8 @@
 #include <time.h>
 #include <gpgme.h>
 
+#include "seahorse-context.h"
+
 typedef enum {
 	SEAHORSE_CRYPT_SUFFIX,
 	SEAHORSE_SIG_SUFFIX,
@@ -39,6 +41,10 @@ typedef enum {
     SEAHORSE_TEXT_TYPE_SIGNED
 } SeahorseTextType;
 
+#define SEAHORSE_EXT_ASC ".asc"
+#define SEAHORSE_EXT_SIG ".sig"
+#define SEAHORSE_EXT_GPG ".gpg"
+
 gchar*		seahorse_util_get_date_string		(const time_t		time);
 
 #define SEAHORSE_GPGME_ERROR  (seahorse_util_gpgme_error_domain())
@@ -50,10 +56,10 @@ void        seahorse_util_gpgme_to_error        (gpgme_error_t gerr,
 void        seahorse_util_show_error            (GtkWindow          *parent,
                                                  const char*        msg);
                                                  
-void        seahorse_util_handle_error          (gpgme_error_t      err,
+void        seahorse_util_handle_gpgme          (gpgme_error_t      err,
                                                  const gchar*       desc, ...);
 
-void        seahorse_util_handle_gerror         (GError*            err,
+void        seahorse_util_handle_error          (GError*            err,
                                                  const char*        desc, ...);
                                                  
 gpgme_error_t	seahorse_util_write_data_to_file	(const gchar		*path,
@@ -85,24 +91,52 @@ const gchar* seahorse_util_uri_split_last   (gchar* uri);
 
 gchar**     seahorse_util_uris_expand       (const gchar **uris);
 
+gboolean    seahorse_util_uris_package      (const gchar* package, 
+                                             const gchar** uris);
+
+GtkWidget*  seahorse_util_chooser_open_new              (const gchar *title,
+                                                         GtkWindow *parent);
+
+GtkWidget*  seahorse_util_chooser_save_new              (const gchar *title,
+                                                         GtkWindow *parent);
+
+void        seahorse_util_chooser_show_key_files        (GtkWidget *dialog);
+
+void        seahorse_util_chooser_show_archive_files    (GtkWidget *dialog);
+                                                 
+gchar*      seahorse_util_chooser_open_prompt           (GtkWidget *dialog);
+
+gchar*      seahorse_util_chooser_save_prompt           (GtkWidget *dialog);
+
 gboolean	seahorse_util_check_suffix		(const gchar		*path,
-							 SeahorseSuffix		suffix);
+                                             SeahorseSuffix     suffix);
 
 gchar*		seahorse_util_add_suffix		(gpgme_ctx_t		ctx,
-							 const gchar		*path,
-							 SeahorseSuffix		suffix);
+                                             const gchar        *path,
+                                             SeahorseSuffix     suffix,
+                                             const gchar        *prompt);
 
-gchar*		seahorse_util_remove_suffix		(const gchar		*path);
+gchar*      seahorse_util_remove_suffix     (const gchar        *path,
+                                             const gchar        *prompt);
 
 gchar**     seahorse_util_strvec_dup        (const gchar        **vec);
 
-gpgme_key_t* seahorse_util_list_to_keys      (GList *keys);
+gpgme_key_t* seahorse_util_keylist_to_keys    (GList *keys);
 
-void         seahorse_util_free_keys         (gpgme_key_t* keys);
+GList*       seahorse_util_keylist_sort       (GList *keys);
 
-/* For checking GPG error codes */
-#define GPG_IS_OK(e)        (gpgme_err_code (e) == GPG_ERR_NO_ERROR)
-#define GPG_OK              (gpgme_error (GPG_ERR_NO_ERROR))
-#define GPG_E(e)            (gpgme_error (e))
+GList*       seahorse_util_keylist_splice     (GList *keys);
+
+void         seahorse_util_free_keys          (gpgme_key_t* keys);
+
+GSList*     seahorse_util_string_slist_free   (GSList *slist);
+
+GSList*     seahorse_util_string_slist_copy   (GSList *slist);
+
+#define     seahorse_util_wait_until(expr)          \
+    while (!(expr)) {                               \
+        g_thread_yield ();                          \
+        g_main_context_iteration (NULL, FALSE);     \
+    }                                               
 
 #endif /* __SEAHORSE_UTIL_H__ */
