@@ -21,8 +21,10 @@
 
 #include <config.h>
 #include <gnome.h>
+#include <libgnomevfs/gnome-vfs.h>
 #include <locale.h>
 
+#include "seahorse-gpgmex.h"
 #include "seahorse-context.h"
 #include "seahorse-windows.h"
 #include "seahorse-op.h"
@@ -122,22 +124,26 @@ read_uri_arguments (GnomeProgram *program)
 static guint
 do_import (SeahorseContext *sctx, const gchar **paths)
 {
+    SeahorseKeySource *sksrc;
     GtkWidget *dlg;
-    gpgme_error_t err;
+    GError *err = NULL;
     gint keys = 0;
     gchar **uris;
     gchar **u;
     gchar *t;
     guint ret = 0;
     
+    sksrc = seahorse_context_get_key_source (sctx);
+    g_return_val_if_fail (sksrc != NULL, 1);
+    
     /* Get all sub-folders etc... */
     uris = seahorse_util_uris_expand (paths);
     g_assert (uris != NULL);
      
     for (u = uris; *u; u++) {
-        keys += seahorse_op_import_file (sctx, *u, &err);
+        keys += seahorse_op_import_file (sksrc, *u, &err);
        
-        if (!GPG_IS_OK (err)) {
+        if (err != NULL) {
             seahorse_util_handle_error (err, _("Couldn't import keys from \"%s\""),
                     seahorse_util_uri_get_last (*u));
             ret = 1;
@@ -160,37 +166,52 @@ do_import (SeahorseContext *sctx, const gchar **paths)
 
 /* Encrypt or sign the given set of paths */
 static guint 
-do_encrypt_base (SeahorseContext *sctx, const gchar **paths, SeahorseEncryptFunc func)
+do_encrypt_base (SeahorseContext *sctx, const gchar **paths, gboolean sign)
 {
-    gpgme_key_t *recips = NULL;
+    SeahorseKeyPair *signer = NULL;
+    SeahorseKeySource *sksrc;
+    GList *keys = NULL;
     gchar *new_path;
     gpgme_error_t err;
     gchar **uris = NULL;
     gchar **u;
     guint ret = 0;
 
+    sksrc = seahorse_context_get_key_source (sctx);
+    g_return_val_if_fail (sksrc != NULL, 1);
+    
     /* This can change the list of uris and optionally compress etc... */
     uris = seahorse_process_multiple (sctx, paths, NULL);
 
     if(!uris)
         ret = 1;
     else {
-        recips = seahorse_recipients_get (sctx);
+        keys = seahorse_recipients_get (sctx, sign ? &signer : NULL);
            
-        if (recips != NULL) {
+        if (g_list_length (keys) > 0) {
             for(u = uris; *u; u++) {
-                new_path = func (sctx, *u, recips, &err);
-                g_free (new_path);            
+               
+                new_path = seahorse_util_add_suffix (sksrc->ctx, *u, SEAHORSE_CRYPT_SUFFIX, 
+                                                     _("Choose Encrypted File Name for '%s'"));
+                if (!new_path) 
+                    break;
+                    
+                if (signer)
+                    seahorse_op_encrypt_sign_file (keys, signer, *u, new_path, &err);
+                else
+                    seahorse_op_encrypt_file (keys, *u, new_path, &err);
+                
+                g_free (new_path);
     
                 if (!GPG_IS_OK (err)) {
-                    seahorse_util_handle_error (err, _("Couldn't encrypt \"%s\""), 
+                    seahorse_util_handle_gpgme (err, _("Couldn't encrypt \"%s\""), 
                         seahorse_util_uri_get_last (*u));
                     ret = 1;
                     break;
                 }
             }
             
-            seahorse_util_free_keys (recips);
+            g_list_free (keys);
        }
 
         g_strfreev (uris);
@@ -203,34 +224,49 @@ do_encrypt_base (SeahorseContext *sctx, const gchar **paths, SeahorseEncryptFunc
 static guint
 do_encrypt (SeahorseContext *sctx, const gchar **paths)
 {
-    return do_encrypt_base (sctx, paths, seahorse_op_encrypt_file);
+    return do_encrypt_base (sctx, paths, FALSE);
 }
 
 /* Encrypt and sign the given set of paths */
 static guint
 do_encrypt_sign (SeahorseContext *sctx, const gchar **paths)
 {
-    return do_encrypt_base (sctx, paths, seahorse_op_encrypt_sign_file);
+    return do_encrypt_base (sctx, paths, TRUE);
 }
 
 static guint
 do_sign (SeahorseContext *sctx, const gchar **paths)
 {
+    SeahorseKeySource *sksrc;
+    SeahorseKeyPair *signer;
     gpgme_error_t err;
     gchar **uris;
     gchar **u;
     gchar *new_path;
     guint ret = 0;
     
+    sksrc = seahorse_context_get_key_source (sctx);
+    g_return_val_if_fail (sksrc != NULL, 1);
+
+    signer = seahorse_signer_get (sctx);    
+    if (signer == NULL)
+        return 1;
+    
     uris = seahorse_util_uris_expand (paths);
     g_assert (uris != NULL);
      
     for (u = uris; *u; u++) {
-        new_path = seahorse_op_sign_file (sctx, *u, &err);
+      
+        new_path = seahorse_util_add_suffix (sksrc->ctx, *u, SEAHORSE_SIG_SUFFIX, 
+                                             _("Choose Signature File Name for '%s'"));
+        if (!new_path)
+            break;
+        
+        seahorse_op_sign_file (signer, *u, new_path, &err);
         g_free(new_path);
         
         if (!GPG_IS_OK (err)) {
-            seahorse_util_handle_error (err, _("Couldn't sign \"%s\""),
+            seahorse_util_handle_gpgme (err, _("Couldn't sign \"%s\""),
                 seahorse_util_uri_get_last (*u));
             ret = 1;
             break;
@@ -245,6 +281,7 @@ do_sign (SeahorseContext *sctx, const gchar **paths)
 static guint
 do_decrypt (SeahorseContext *sctx, const gchar **paths)
 {
+    SeahorseKeySource *sksrc;
     gpgme_verify_result_t status = NULL;
     SeahorseWidget *signatures = NULL;
     gpgme_error_t err;
@@ -252,20 +289,28 @@ do_decrypt (SeahorseContext *sctx, const gchar **paths)
     gchar **u;
     gchar *new_path;
     guint ret = 0;
-    
+	
+    sksrc = seahorse_context_get_key_source (sctx);
+    g_return_val_if_fail (sksrc != NULL, 1);
+        
     uris = seahorse_util_uris_expand (paths);
     g_assert (uris != NULL);
      
     for (u = uris; *u; u++) {
-        new_path = seahorse_op_decrypt_verify_file (sctx, *u, &status, &err);
+      
+        new_path = seahorse_util_remove_suffix (*u, _("Choose Decrypted File Name for '%s'"));
+        if(!new_path)
+            break;
+            
+        seahorse_op_decrypt_verify_file (sksrc, *u, new_path, &status, &err);
                 
         if (!GPG_IS_OK (err)) {
-            seahorse_util_handle_error (err, _("Couldn't decrypt \"%s\""),
+            seahorse_util_handle_gpgme (err, _("Couldn't decrypt \"%s\""),
                     seahorse_util_uri_get_last (*u));
             ret = 1;
             break;
         }
-        
+    
         if(status && status->signatures) {
             if(!signatures)
                 signatures = seahorse_signatures_new (sctx);
@@ -287,6 +332,7 @@ do_decrypt (SeahorseContext *sctx, const gchar **paths)
 static guint
 do_verify (SeahorseContext *sctx, const gchar **paths)
 {
+    SeahorseKeySource *sksrc;
     gpgme_verify_result_t status = NULL;
     SeahorseWidget *signatures = NULL;
     gchar *original = NULL;
@@ -297,10 +343,13 @@ do_verify (SeahorseContext *sctx, const gchar **paths)
     
     uris = seahorse_util_uris_expand (paths);
     g_assert (uris != NULL);
-     
+	
+    sksrc = seahorse_context_get_key_source (sctx);
+    g_return_val_if_fail (sksrc != NULL, 1);
+         
     for (u = uris; *u; u++) {
 
-        original = seahorse_util_remove_suffix (*u);
+        original = seahorse_util_remove_suffix (*u, NULL);
 
         /* The original file doesn't exist, prompt for it */
         if (!seahorse_util_uri_exists (original)) {
@@ -332,11 +381,11 @@ do_verify (SeahorseContext *sctx, const gchar **paths)
         }
         
         if (original) {
-            seahorse_op_verify_file (sctx, *u, original, &status, &err);
+            seahorse_op_verify_file (sksrc, *u, original, &status, &err);
             g_free (original);
 
             if (!GPG_IS_OK (err)) {
-                seahorse_util_handle_error (err, _("Couldn't verify \"%s\""),
+                seahorse_util_handle_gpgme (err, _("Couldn't verify \"%s\""),
                         seahorse_util_uri_get_last (*u));
                 ret = 1;
                 break;
@@ -386,6 +435,7 @@ main (int argc, char **argv)
                     GNOME_PARAM_APP_DATADIR, DATA_DIR, NULL);
 
     sctx = seahorse_context_new ();
+    seahorse_context_load_keys (sctx, FALSE);
 
     if (mode != MODE_NONE) {
         gchar **uris = NULL;
@@ -420,13 +470,15 @@ main (int argc, char **argv)
                 
             g_strfreev (uris);
         }
-        
-        return ret;
-    }
- 
-    win = seahorse_key_manager_show (sctx);
-    g_signal_connect_after (G_OBJECT (win), "destroy", gtk_main_quit, NULL);
-	gtk_main ();
-	
-	return 0;
+
+    } else { 
+        win = seahorse_key_manager_show (sctx);
+        g_signal_connect_after (G_OBJECT (win), "destroy", gtk_main_quit, NULL);
+	    gtk_main ();
+	}
+    
+    if (gnome_vfs_initialized ())
+        gnome_vfs_shutdown ();
+    
+	return ret;
 }
