@@ -25,9 +25,12 @@
 #include <eel/eel-gconf-extensions.h>
 #include <libgnomevfs/gnome-vfs.h>
 
+#include "seahorse-gpgmex.h"
 #include "seahorse-windows.h"
 #include "seahorse-widget.h"
+#include "seahorse-progress.h"
 #include "seahorse-preferences.h"
+#include "seahorse-operation.h"
 #include "seahorse-util.h"
 #include "seahorse-validity.h"
 #include "seahorse-key-manager-store.h"
@@ -35,11 +38,12 @@
 #include "seahorse-key-op.h"
 #include "seahorse-key-widget.h"
 #include "seahorse-op.h"
+#include "seahorse-gpg-options.h"
 
 #define KEY_LIST "key_list"
 
-static guint signal_id = 0;
-static gulong hook_id = 0;
+#define SEC_RING "/secring.gpg"
+#define PUB_RING "/pubring.gpg"
 
 /* Drag and trop target types */
 enum TargetTypes {
@@ -57,7 +61,6 @@ static const GtkTargetEntry target_entries[] = {
 static void
 quit (GtkWidget *widget, SeahorseWidget *swidget)
 {
-	g_signal_remove_emission_hook (signal_id, hook_id);
 	seahorse_context_destroy (swidget->sctx);
 }
 
@@ -82,24 +85,6 @@ new_button_clicked (GtkWidget *widget, SeahorseWidget *swidget)
 	seahorse_generate_druid_show (swidget->sctx);
 }
 
-/* Setup our file types on a file chooser dialog */
-static void
-setup_file_types (GtkFileChooser* dialog)
-{
-    GtkFileFilter* filter = gtk_file_filter_new ();
-    gtk_file_filter_set_name (filter, _("All PGP key files"));
-    gtk_file_filter_add_pattern (filter, "*.asc");    
-    gtk_file_filter_add_pattern (filter, "*.key");    
-    gtk_file_filter_add_pattern (filter, "*.pkr");    
-    gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (dialog), filter);    
-    gtk_file_chooser_set_filter (GTK_FILE_CHOOSER (dialog), filter);
-
-    filter = gtk_file_filter_new ();
-    gtk_file_filter_set_name (filter, _("All files"));
-    gtk_file_filter_add_pattern (filter, "*");    
-    gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (dialog), filter);    
-}
-
 /* Loads import dialog */
 static void
 import_activate (GtkWidget *widget, SeahorseWidget *swidget)
@@ -107,33 +92,21 @@ import_activate (GtkWidget *widget, SeahorseWidget *swidget)
     SeahorseKeySource *sksrc;
     GtkWidget *dialog;
     char* uri = NULL;
+    GError *err = NULL;
     gint keys;
-    gpgme_error_t err;
     
-    sksrc = seahorse_context_get_pri_source (swidget->sctx);
+    sksrc = seahorse_context_get_key_source (swidget->sctx);
     g_return_if_fail (sksrc != NULL);
     
-    dialog = gtk_file_chooser_dialog_new (_("Import Key"), 
-                GTK_WINDOW(glade_xml_get_widget (swidget->xml, "key-manager")),
-                GTK_FILE_CHOOSER_ACTION_OPEN, 
-                GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-                GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
-                NULL);
-
-    setup_file_types (GTK_FILE_CHOOSER (dialog));
-    gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (dialog), FALSE);
-     
-    if(gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
-        uri = gtk_file_chooser_get_uri (GTK_FILE_CHOOSER (dialog));
-        
-    gtk_widget_destroy (dialog);
+    dialog = seahorse_util_chooser_open_new (_("Import Key"), 
+                GTK_WINDOW(glade_xml_get_widget (swidget->xml, "key-manager")));
+    seahorse_util_chooser_show_key_files (dialog);
 
+    uri = seahorse_util_chooser_open_prompt (dialog);
     if(uri) {
         keys = seahorse_op_import_file (sksrc, uri, &err);
         
-        if (GPG_IS_OK (err))
-            seahorse_key_source_refresh (sksrc, FALSE);
-        else
+        if (err != NULL)
             seahorse_util_handle_error (err, _("Couldn't import keys from \"%s\""), 
                 seahorse_util_uri_get_last (uri));
                 
@@ -146,18 +119,16 @@ static void
 clipboard_received (GtkClipboard *board, const gchar *text, SeahorseContext *sctx)
 {
     SeahorseKeySource *sksrc;
-    gpgme_error_t err;
+    GError *err = NULL;
     gint keys;
     
-    sksrc = seahorse_context_get_pri_source (sctx);
+    sksrc = seahorse_context_get_key_source (sctx);
     g_return_if_fail (sksrc != NULL);
  
     keys = seahorse_op_import_text (sksrc, text, &err);
  
-    if (!GPG_IS_OK (err))
+    if (err != NULL)
         seahorse_util_handle_error (err, _("Couldn't import keys from clipboard"));
-    else if (keys > 0)
-        seahorse_key_source_refresh (sksrc, FALSE);
 }
 
 /* Pastes key from keyboard */
@@ -180,7 +151,7 @@ copy_activate (GtkWidget *widget, SeahorseWidget *swidget)
     GdkAtom atom;
     GtkClipboard *board;
     gchar *text;
-    gpgme_error_t err;
+    GError *err = NULL;
     GList *keys;
   
     keys = seahorse_key_store_get_selected_keys (GTK_TREE_VIEW (
@@ -191,9 +162,9 @@ copy_activate (GtkWidget *widget, SeahorseWidget *swidget)
                
     text = seahorse_op_export_text (keys, &err);
 
-    if (!GPG_IS_OK (err))
+    if (text == NULL)
         seahorse_util_handle_error (err, _("Couldn't export key to clipboard"));
-    else if (text != NULL) {
+    else {
         atom = gdk_atom_intern ("CLIPBOARD", FALSE);
         board = gtk_clipboard_get (atom);
         gtk_clipboard_set_text (board, text, strlen (text));
@@ -208,7 +179,7 @@ properties_activate (GtkWidget *widget, SeahorseWidget *swidget)
 	SeahorseKey *skey;
 	
 	skey = seahorse_key_store_get_selected_key (GTK_TREE_VIEW (
-		glade_xml_get_widget (swidget->xml, KEY_LIST)));
+		glade_xml_get_widget (swidget->xml, KEY_LIST)), NULL);
 	if (skey != NULL)
 		seahorse_key_properties_new (swidget->sctx, skey);
 }
@@ -218,54 +189,15 @@ static void
 export_activate (GtkWidget *widget, SeahorseWidget *swidget)
 {
     GtkWidget *dialog;
-    char* uri = NULL;
-    gpgme_error_t err;
+    gchar* uri = NULL;
+    GError *err = NULL;
     GList *keys;
     
-    dialog = gtk_file_chooser_dialog_new (_("Export Key"), 
-                GTK_WINDOW(glade_xml_get_widget (swidget->xml, "key-manager")),
-                GTK_FILE_CHOOSER_ACTION_SAVE, 
-                GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-                GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
-                NULL);
-
-    setup_file_types (GTK_FILE_CHOOSER(dialog));
-    gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (dialog), FALSE);
+    dialog = seahorse_util_chooser_save_new (_("Export Key"), 
+                GTK_WINDOW(glade_xml_get_widget (swidget->xml, "key-manager")));
+    seahorse_util_chooser_show_key_files (dialog);
      
-    while(gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT) {
-        GnomeVFSURI *u = gnome_vfs_uri_new (gtk_file_chooser_get_uri(GTK_FILE_CHOOSER(dialog)));
-        if (u != NULL) {
-            if (gnome_vfs_uri_exists(u)) {
-                GtkWidget* edlg;
-                edlg = gtk_message_dialog_new_with_markup (GTK_WINDOW (dialog),
-                    GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_QUESTION,
-                    GTK_BUTTONS_NONE, _("<b>A file already exists with this name.</b>\n\nDo you want to replace it with the one you are saving?"));
-                gtk_dialog_add_buttons (GTK_DIALOG (edlg), 
-                       GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-                       _("Replace"), GTK_RESPONSE_ACCEPT, NULL);
-
-                gtk_dialog_set_default_response (GTK_DIALOG (edlg), GTK_RESPONSE_CANCEL);                    
-                
-                if (gtk_dialog_run (GTK_DIALOG (edlg)) == GTK_RESPONSE_ACCEPT)
-                    uri = gtk_file_chooser_get_uri (GTK_FILE_CHOOSER (dialog));
-
-                gtk_widget_destroy (edlg);
-
-            /* File doesn't exist */
-            } else {
-                uri = gtk_file_chooser_get_uri (GTK_FILE_CHOOSER (dialog));
-            }
-            
-            gnome_vfs_uri_unref (u);
-
-        }
-
-        if (uri != NULL)
-            break;
-    }
-
-    gtk_widget_destroy (dialog);
-
+    uri = seahorse_util_chooser_save_prompt (dialog);
     if(uri) {
         keys = seahorse_key_store_get_selected_keys (GTK_TREE_VIEW (
             glade_xml_get_widget (swidget->xml, KEY_LIST)));
@@ -273,7 +205,7 @@ export_activate (GtkWidget *widget, SeahorseWidget *swidget)
         seahorse_op_export_file (keys, uri, &err); 
 		g_list_free (keys);
 		        
-        if (!GPG_IS_OK (err))
+        if (err != NULL)
             seahorse_util_handle_error (err, _("Couldn't export key to \"%s\""),
                     seahorse_util_uri_get_last (uri));
                     
@@ -281,6 +213,51 @@ export_activate (GtkWidget *widget, SeahorseWidget *swidget)
     }
 }
 
+/*Archives public and private keyrings*/
+static void
+backup_activate (GtkWidget *widget, SeahorseWidget *swidget)
+{
+    GtkWidget *dialog;
+	gchar *uri = NULL;
+    gchar **uris;
+    gchar *ext, *t;
+    const gchar* home_dir = NULL;
+    
+    dialog = seahorse_util_chooser_save_new (_("Backup Keyrings to Archive"), 
+                GTK_WINDOW(glade_xml_get_widget (swidget->xml, "key-manager")));
+    seahorse_util_chooser_show_archive_files (dialog);	    	    	    
+
+    uri = seahorse_util_chooser_save_prompt (dialog);
+    if(uri)  {	    	
+       
+        /* Save the extension */
+        t = strchr(uri, '.');
+        if(t != NULL) {
+            t++;
+            if(t[0] != 0) 
+                eel_gconf_set_string (MULTI_EXTENSION_KEY, t);
+        } else {
+            if ((ext = eel_gconf_get_string (MULTI_EXTENSION_KEY)) == NULL)
+    	        ext = g_strdup ("zip"); /* Yes this happens when the schema isn't installed */
+	        
+        	t = seahorse_util_uri_replace_ext (uri, ext);
+        	g_free(uri);
+        	g_free(ext);
+        	uri = t;
+        }
+	        
+    	home_dir = (const gchar*)seahorse_gpg_homedir();
+	    	
+        uris = g_new0 (gchar*, 3);
+    	uris[0] = g_strconcat (home_dir, PUB_RING, NULL);
+    	uris[1] = g_strconcat (home_dir, SEC_RING, NULL);
+	    	
+    	seahorse_util_uris_package (uri, (const gchar**)uris);
+
+        g_strfreev (uris);
+    }
+}
+
 static void
 sign_activate (GtkWidget *widget, SeahorseWidget *swidget)
 {
@@ -291,15 +268,41 @@ sign_activate (GtkWidget *widget, SeahorseWidget *swidget)
 	seahorse_sign_show (swidget->sctx, list);
 }
 
+#ifdef WITH_KEYSERVER
+static void
+search_activate (GtkWidget *widget, SeahorseWidget *swidget)
+{
+    seahorse_keyserver_search_show (swidget->sctx);
+}
+#endif
+
 /* Loads delete dialog if a key is selected */
 static void
 delete_activate (GtkWidget *widget, SeahorseWidget *swidget)
 {
 	GList *list = NULL;
+    GtkTreeView *view = 
+        GTK_TREE_VIEW (glade_xml_get_widget (swidget->xml, KEY_LIST));
 	
-	list = seahorse_key_store_get_selected_keys (GTK_TREE_VIEW (
-		glade_xml_get_widget (swidget->xml, KEY_LIST)));
-	seahorse_delete_show (swidget->sctx, list);
+	list = seahorse_key_store_get_selected_keys (view);
+    
+    /* Special behavior for a single selection */
+    if (g_list_length (list) == 1) {
+        SeahorseKey *skey;
+        guint uid;
+        
+        skey = seahorse_key_store_get_selected_key (view, &uid);
+        if (uid > 0) 
+            seahorse_delete_userid_show (swidget->sctx, skey, uid);
+        else    
+        	seahorse_delete_show (swidget->sctx, list);
+
+    /* Multiple keys */
+    } else {    
+	    seahorse_delete_show (swidget->sctx, list);
+    }
+    
+  	g_list_free (list);
 }
 
 static void
@@ -308,7 +311,7 @@ change_passphrase_activate (GtkMenuItem *item, SeahorseWidget *swidget)
 	SeahorseKey *skey;
 	
 	skey = seahorse_key_store_get_selected_key (GTK_TREE_VIEW (
-		glade_xml_get_widget (swidget->xml, KEY_LIST)));
+		glade_xml_get_widget (swidget->xml, KEY_LIST)), NULL);
 	if (skey != NULL && SEAHORSE_IS_KEY_PAIR (skey))
 		seahorse_key_pair_op_change_pass (SEAHORSE_KEY_PAIR (skey));
 }
@@ -319,7 +322,7 @@ add_uid_activate (GtkMenuItem *item, SeahorseWidget *swidget)
 	SeahorseKey *skey;
 	
 	skey = seahorse_key_store_get_selected_key (GTK_TREE_VIEW (
-		glade_xml_get_widget (swidget->xml, KEY_LIST)));
+		glade_xml_get_widget (swidget->xml, KEY_LIST)), NULL);
 	if (skey != NULL)
 		seahorse_add_uid_new (swidget->sctx, skey);
 }
@@ -330,7 +333,7 @@ add_subkey_activate (GtkMenuItem *item, SeahorseWidget *swidget)
 	SeahorseKey *skey;
 	
 	skey = seahorse_key_store_get_selected_key (GTK_TREE_VIEW (
-		glade_xml_get_widget (swidget->xml, KEY_LIST)));
+		glade_xml_get_widget (swidget->xml, KEY_LIST)), NULL);
 	if (skey != NULL)
 		seahorse_add_subkey_new (swidget->sctx, skey);
 }
@@ -341,7 +344,7 @@ add_revoker_activate (GtkMenuItem *item, SeahorseWidget *swidget)
 	SeahorseKey *skey;
 	
 	skey = seahorse_key_store_get_selected_key (GTK_TREE_VIEW (
-		glade_xml_get_widget (swidget->xml, KEY_LIST)));
+		glade_xml_get_widget (swidget->xml, KEY_LIST)), NULL);
 	if (skey != NULL)
 		seahorse_add_revoker_new (swidget->sctx, skey);
 }
@@ -377,6 +380,10 @@ about_activate (GtkWidget *widget, SeahorseWidget *swidget)
 		"Jose Carlos Garcia Sogo <jsogo@users.sourceforge.net>",
 		"Jean Schurger <yshark@schurger.org>",
         "Nate Nielsen <nielsen@memberwebs.com>",
+        "",
+        "Contributions:",
+        "Adam Schreiber <sadam@clemson.edu>",
+        "Albrecht Dreß <albrecht.dress@arcor.de>",
 		NULL
 	};
 
@@ -406,8 +413,8 @@ about_activate (GtkWidget *widget, SeahorseWidget *swidget)
 	}
 
 	about = gnome_about_new (_("seahorse"), VERSION,
-		"Copyright \xc2\xa9 2002, 2003 Seahorse Project",
-		"http://seahorse.sourceforge.net",
+		"Copyright \xc2\xa9 2002, 2003, 2004 Seahorse Project",
+		"http://seahorse.sourceforge.net/",
 		(const char **)authors, (const char **)documenters,
 		strcmp (translator_credits, "translator_credits") != 0 ?
 			translator_credits : NULL,
@@ -430,7 +437,7 @@ row_activated (GtkTreeView *treeview, GtkTreePath *path, GtkTreeViewColumn *arg2
 {
 	SeahorseKey *skey;
 	
-	skey = seahorse_key_store_get_key_from_path (GTK_TREE_VIEW (glade_xml_get_widget (swidget->xml, KEY_LIST)), path);
+	skey = seahorse_key_store_get_key_from_path (GTK_TREE_VIEW (glade_xml_get_widget (swidget->xml, KEY_LIST)), path, NULL);
 	if (skey != NULL)
 		seahorse_key_properties_new (swidget->sctx, skey);
 }
@@ -445,7 +452,6 @@ set_key_options_sensitive (SeahorseWidget *swidget, gboolean selected, gboolean
 	gtk_widget_set_sensitive (glade_xml_get_widget (swidget->xml, "sign_button"), create);
 	gtk_widget_set_sensitive (glade_xml_get_widget (swidget->xml, "sign"), create);
 	
-	gtk_widget_set_sensitive (glade_xml_get_widget (swidget->xml, "delete_button"), selected);
 	gtk_widget_set_sensitive (glade_xml_get_widget (swidget->xml, "delete"), selected);
 	gtk_widget_set_sensitive (glade_xml_get_widget (swidget->xml, "export_button"), selected);
 	gtk_widget_set_sensitive (glade_xml_get_widget (swidget->xml, "export"), selected);
@@ -490,7 +496,7 @@ selection_changed (GtkTreeSelection *selection, SeahorseWidget *swidget)
 	
 	if (rows == 1) {
 		skey = seahorse_key_store_get_selected_key (GTK_TREE_VIEW (
-			glade_xml_get_widget (swidget->xml, KEY_LIST)));
+			glade_xml_get_widget (swidget->xml, KEY_LIST)), NULL);
 		secret = (skey != NULL && SEAHORSE_IS_KEY_PAIR (skey));
 	}
 	
@@ -521,7 +527,7 @@ key_list_popup_menu (GtkWidget *widget, SeahorseWidget *swidget)
 {
 	SeahorseKey *skey;
 	
-	skey = seahorse_key_store_get_selected_key (GTK_TREE_VIEW (widget));
+	skey = seahorse_key_store_get_selected_key (GTK_TREE_VIEW (widget), NULL);
 	if (skey != NULL)
 		show_context_menu (swidget, 0, gtk_get_current_event_time ());
        
@@ -544,88 +550,19 @@ gconf_notification (GConfClient *gclient, guint id, GConfEntry *entry, SeahorseW
      */
 }
 
-#if 0
-static void
-show_progress (SeahorseContext *sctx, const gchar *op, gdouble fract, SeahorseWidget *swidget)
-{
-	GnomeAppBar *status;
-	GtkProgressBar *progress;
-	gboolean sensitive;
-
-	sensitive = (fract == -1);
-	
-	status = GNOME_APPBAR (glade_xml_get_widget (swidget->xml, "status"));
-	gnome_appbar_set_status (status, op);
-	progress = gnome_appbar_get_progress (status);
-	/* do progress */
-	if (fract <= 1 && fract > 0)
-		gtk_progress_bar_set_fraction (progress, fract);
-	else if (fract != -1) {
-		gtk_progress_bar_set_pulse_step (progress, 0.05);
-		gtk_progress_bar_pulse (progress);
-	}
-	/* if fract == -1, cleanup progress */
-	else
-		gtk_progress_bar_set_fraction (progress, 0);
-	
-	gtk_widget_set_sensitive (glade_xml_get_widget (swidget->xml, "key"), sensitive);
-	gtk_widget_set_sensitive (glade_xml_get_widget (swidget->xml, "edit"), sensitive);
-	gtk_widget_set_sensitive (glade_xml_get_widget (swidget->xml, "properties_button"), sensitive);
-	gtk_widget_set_sensitive (glade_xml_get_widget (swidget->xml, "export_button"), sensitive);
-	gtk_widget_set_sensitive (glade_xml_get_widget (swidget->xml, "sign_button"), sensitive);
-	gtk_widget_set_sensitive (glade_xml_get_widget (swidget->xml, "delete_button"), sensitive);
-	gtk_widget_set_sensitive (glade_xml_get_widget (swidget->xml, KEY_LIST), sensitive);
-	
-	while (g_main_context_pending (NULL))
-		g_main_context_iteration (NULL, TRUE);
-}
-#endif
-
-/* params[0] = sctx, params[1] = op string, params[2] = fract */
-static gboolean
-progress_hook (GSignalInvocationHint *hint, guint n_params, const GValue *params, SeahorseWidget *swidget)
-{
-	GnomeAppBar *status;
-	GtkProgressBar *progress;
-	/* gboolean sensitive; */
-	gdouble fract;
-
-	fract = g_value_get_double (&params[2]);
-	/* sensitive = (fract == -1); */
-	
-	status = GNOME_APPBAR (glade_xml_get_widget (swidget->xml, "status"));
-	gnome_appbar_set_status (status, g_value_get_string (&params[1]));
-	progress = gnome_appbar_get_progress (status);
-	/* do progress */
-	if (fract <= 1 && fract > 0)
-		gtk_progress_bar_set_fraction (progress, fract);
-	else if (fract != -1) {
-		gtk_progress_bar_set_pulse_step (progress, 0.05);
-		gtk_progress_bar_pulse (progress);
-	}
-	/* if fract == -1, cleanup progress */
-	else
-		gtk_progress_bar_set_fraction (progress, 0);
-	
-	while (g_main_context_pending (NULL))
-		g_main_context_iteration (NULL, TRUE);
-	
-	return TRUE;
-}
-
 static void
 target_drag_data_received (GtkWidget *widget, GdkDragContext *context, gint x, gint y,
                            GtkSelectionData *data, guint info, guint time, SeahorseContext *sctx)
 {
     SeahorseKeySource *sksrc;
     gint keys = 0;
-    gpgme_error_t err;
+    GError *err = NULL;
     gchar** uris;
     gchar** u;
     
     g_return_if_fail (data != NULL);
     
-    sksrc = seahorse_context_get_pri_source (sctx);
+    sksrc = seahorse_context_get_key_source (sctx);
     g_return_if_fail (sksrc != NULL);
     
     switch(info) {
@@ -639,7 +576,7 @@ target_drag_data_received (GtkWidget *widget, GdkDragContext *context, gint x, g
             g_strstrip (*u);
             if ((*u)[0]) { /* Make sure it's not an empty line */
                 keys += seahorse_op_import_file (sksrc, *u, &err);  
-                if (!GPG_IS_OK (err)) {
+                if (err != NULL) {
                     seahorse_util_handle_error (err, _("Couldn't import key from \"%s\""),
                             seahorse_util_uri_get_last (*u));
                     break;
@@ -653,9 +590,6 @@ target_drag_data_received (GtkWidget *widget, GdkDragContext *context, gint x, g
         g_assert_not_reached ();
         return;
     } 
-
-    if (keys > 0)
-        seahorse_key_source_refresh (sksrc, FALSE);        
 }
 
 /* Refilter the keys when the filter text changes */
@@ -669,15 +603,23 @@ filter_changed (GtkWidget *widget, SeahorseKeyStore* skstore)
 GtkWindow* 
 seahorse_key_manager_show (SeahorseContext *sctx)
 {
+    GtkWindow *win;
+    SeahorseOperation *operation;
 	SeahorseWidget *swidget;
 	GtkTreeView *view;
     GtkWidget* w;
 	GtkTreeSelection *selection;
     SeahorseKeyStore *skstore;
+    SeahorseKeySource *sksrc;
 	
 	swidget = seahorse_widget_new ("key-manager", sctx);
 	gtk_object_sink (GTK_OBJECT (sctx));
-	
+
+    win = GTK_WINDOW (glade_xml_get_widget (swidget->xml, "key-manager"));
+
+    sksrc = seahorse_context_get_key_source (sctx);
+    g_return_val_if_fail (sksrc != NULL, win);
+    	
 	/* construct key context menu */
 	glade_xml_construct (swidget->xml, SEAHORSE_GLADEDIR "seahorse-key-manager.glade",
 		"context_menu", NULL);
@@ -686,10 +628,11 @@ seahorse_key_manager_show (SeahorseContext *sctx)
 	//features not available
 	gtk_widget_set_sensitive (glade_xml_get_widget (swidget->xml, "add_photo"), FALSE);
 	gtk_widget_set_sensitive (glade_xml_get_widget (swidget->xml, "key_add_photo"), FALSE);
-	gtk_widget_set_sensitive (glade_xml_get_widget (swidget->xml, "backup"), FALSE);
-	gtk_widget_set_sensitive (glade_xml_get_widget (swidget->xml, "key_backup"), FALSE);
 	gtk_widget_set_sensitive (glade_xml_get_widget (swidget->xml, "gen_revoke"), FALSE);
 	gtk_widget_set_sensitive (glade_xml_get_widget (swidget->xml, "key_gen_revoke"), FALSE);
+
+    glade_xml_signal_connect_data (swidget->xml, "backup_activate",
+        G_CALLBACK (backup_activate), swidget);
 	
 	/* quit signals */
 	glade_xml_signal_connect_data (swidget->xml, "quit",
@@ -702,7 +645,16 @@ seahorse_key_manager_show (SeahorseContext *sctx)
 		G_CALLBACK (generate_activate), swidget);
 	glade_xml_signal_connect_data (swidget->xml, "import_activate",
 		G_CALLBACK (import_activate), swidget);
-	
+      
+    /* Keyserver stuff */
+#ifdef WITH_KEYSERVER      
+    glade_xml_signal_connect_data (swidget->xml, "search_activate",
+                                   G_CALLBACK (search_activate), swidget);
+#else
+    gtk_widget_hide (glade_xml_get_widget (swidget->xml, "search_button"));    
+    gtk_widget_hide (glade_xml_get_widget (swidget->xml, "keyserver_search"));    
+#endif	
+
 	glade_xml_signal_connect_data (swidget->xml, "expand_all_activate",
 		G_CALLBACK (expand_all_activate), swidget);
 	glade_xml_signal_connect_data (swidget->xml, "collapse_all_activate",
@@ -736,14 +688,8 @@ seahorse_key_manager_show (SeahorseContext *sctx)
 	glade_xml_signal_connect_data (swidget->xml, "add_revoker_activate",
 		G_CALLBACK (add_revoker_activate), swidget);
 	
-	//g_signal_connect (swidget->sctx, "progress", G_CALLBACK (show_progress), swidget);
-	signal_id = g_signal_lookup ("progress", G_OBJECT_CLASS_TYPE (G_OBJECT_CLASS (SEAHORSE_CONTEXT_GET_CLASS (sctx))));
-	hook_id = g_signal_add_emission_hook (signal_id, 0, (GSignalEmissionHook)progress_hook,
-		swidget, (GDestroyNotify)seahorse_widget_destroy);
-	
 	/* init gclient */
 	eel_gconf_notification_add (UI_SCHEMAS, (GConfClientNotifyFunc) gconf_notification, swidget);
-	eel_gconf_monitor_add (UI_SCHEMAS);
 	
 	/* first time signals */
 	glade_xml_signal_connect_data (swidget->xml, "import_button_clicked",
@@ -765,27 +711,32 @@ seahorse_key_manager_show (SeahorseContext *sctx)
 	gtk_tree_selection_set_mode (selection, GTK_SELECTION_MULTIPLE);
 	g_signal_connect (selection, "changed",
 		G_CALLBACK (selection_changed), swidget);
-    skstore = seahorse_key_manager_store_new (sctx, view);
+    skstore = seahorse_key_manager_store_new (sksrc, view);
 	selection_changed (selection, swidget);
    
-    w = glade_xml_get_widget (swidget->xml, "key-manager");
-    
     /* Setup drops */
-    gtk_drag_dest_set (w, GTK_DEST_DEFAULT_ALL, target_entries, 
+    gtk_drag_dest_set (GTK_WIDGET (win), GTK_DEST_DEFAULT_ALL, target_entries, 
             sizeof (target_entries) / sizeof (target_entries[0]), GDK_ACTION_COPY);
-    gtk_signal_connect (GTK_OBJECT (w), "drag_data_received",
+    gtk_signal_connect (GTK_OBJECT (win), "drag_data_received",
             GTK_SIGNAL_FUNC (target_drag_data_received), sctx);
                         
     /* For the filtering */
     glade_xml_signal_connect_data(swidget->xml, "on_filter_changed",
                               G_CALLBACK(filter_changed), skstore);
 
+    /* Hook progress bar in */
+    operation = seahorse_key_source_get_operation (sksrc);
+    g_return_val_if_fail (operation != NULL, win);
+    
+    w = glade_xml_get_widget (swidget->xml, "status");
+    seahorse_progress_appbar_add_operation (w, operation);
+    
     /* Although not all the keys have completed we'll know whether we have 
      * any or not at this point */
-	if (seahorse_context_get_n_keys (sctx) == 0) {
+	if (seahorse_key_source_get_count (sksrc, FALSE) == 0) {
 		w = glade_xml_get_widget (swidget->xml, "first-time-box");
 		gtk_widget_show (w);
 	}
 	
-    return GTK_WINDOW (w);
+    return win;
 }
