@@ -22,6 +22,7 @@
 #include <gnome.h>
 #include <time.h>
 
+#include "seahorse-gpgmex.h"
 #include "seahorse-key-dialogs.h"
 #include "seahorse-key-widget.h"
 #include "seahorse-util.h"
@@ -162,6 +163,81 @@ do_stat_button (gchar *label, const gchar *stock_id)
 	return container;
 }
 
+static void
+key_property_labels (SeahorseWidget *swidget)
+{
+    gchar dbuffer[G_ASCII_DTOSTR_BUF_SIZE];
+    gpgme_subkey_t subkey;
+    SeahorseKey *skey;
+    GtkWidget *w;
+    const gchar *label;
+    gchar *t, *x;
+
+    skey = SEAHORSE_KEY_WIDGET (swidget)->skey;
+    subkey = skey->key->subkeys;
+    
+    w = glade_xml_get_widget (swidget->xml, "keyid");
+    label = seahorse_key_get_keyid (skey, 0); 
+    gtk_label_set_text (GTK_LABEL (w), label);
+    
+    w = glade_xml_get_widget (swidget->xml, "fingerprint");
+    label = seahorse_key_get_fingerprint (skey);  
+    gtk_label_set_text (GTK_LABEL (w), label);
+
+    w = glade_xml_get_widget (swidget->xml, "label_type");
+    label = gpgme_pubkey_algo_name(subkey->pubkey_algo);
+
+    if (label == NULL)
+        label = _("Unknown");
+    else if (g_str_equal ("ElG", label))
+        label = _("ElGamal");
+    gtk_label_set_text (GTK_LABEL (w), label);
+    
+    w = glade_xml_get_widget (swidget->xml, "label_created");
+    label = seahorse_util_get_date_string (subkey->timestamp); 
+    gtk_label_set_text (GTK_LABEL (w), label);
+
+    w = glade_xml_get_widget (swidget->xml, "label_strength");
+    g_ascii_dtostr(dbuffer, G_ASCII_DTOSTR_BUF_SIZE, subkey->length);
+    gtk_label_set_text (GTK_LABEL (w), dbuffer);
+    
+    w = glade_xml_get_widget (swidget->xml, "label_expires");
+    label = seahorse_util_get_date_string (subkey->expires);
+    if (g_str_equal("0", label))
+	    label = _("Never");
+    gtk_label_set_text (GTK_LABEL (w), label);
+    
+    w = glade_xml_get_widget (swidget->xml, "label_created");
+    label = seahorse_util_get_date_string (subkey->timestamp); 
+    gtk_label_set_text (GTK_LABEL (w), label);
+    
+    if ((t = seahorse_key_get_userid_name (skey, 0)) != NULL) {
+        w = glade_xml_get_widget (swidget->xml, "label_name");
+        gtk_label_set_text (GTK_LABEL (w), t);  
+        g_free (t);
+    }
+    
+    if ((t = seahorse_key_get_userid_email (skey, 0)) != NULL) {
+        w = glade_xml_get_widget (swidget->xml, "label_email");
+        gtk_label_set_text (GTK_LABEL (w), t);  
+        g_free (t);
+    }
+    
+    if ((t = seahorse_key_get_userid_comment (skey, 0)) != NULL) {
+        w = glade_xml_get_widget (swidget->xml, "label_comment");
+        gtk_label_set_text (GTK_LABEL (w), t);  
+        g_free (t);
+    }
+
+    t = seahorse_key_get_userid (skey, 0);
+    x = g_strdup_printf (_("%s Properties"), t);
+    g_free (t);
+    
+    w = glade_xml_get_widget (swidget->xml, swidget->name);
+    gtk_window_set_title (GTK_WINDOW (w), x);
+    g_free (x);
+}
+
 static void
 do_stats (SeahorseWidget *swidget, GtkTable *table, guint top, guint index, gpgme_subkey_t subkey)
 {
@@ -182,7 +258,9 @@ do_stats (SeahorseWidget *swidget, GtkTable *table, guint top, guint index, gpgm
     /* type */
     do_stat_label (_("Type:"), table, 2, top, FALSE, tips, _("Algorithm"));
     str = gpgme_pubkey_algo_name(subkey->pubkey_algo);
-    if (g_str_equal ("ElG", str))
+    if (str == NULL)
+        str = "Unknown";
+    else if (g_str_equal ("ElG", str))
         str = "ElGamal";
     do_stat_label (str, table, 3, top, FALSE, NULL, NULL);
     /* created */
@@ -203,14 +281,14 @@ do_stats (SeahorseWidget *swidget, GtkTable *table, guint top, guint index, gpgm
         str = _("Expired");
     do_stat_label (str, table, 1, top+3, FALSE, NULL, NULL);
     /* expires */
-    do_stat_label (_("Expiration Date:"), table, 0, top+2, FALSE, NULL, NULL);
+    do_stat_label (_("Expires:"), table, 0, top+2, FALSE, NULL, NULL);
  
    if (!SEAHORSE_IS_KEY_PAIR (skey)) {
         if (subkey->expires)
             do_stat_label (seahorse_util_get_date_string (subkey->expires),
                 table, 1, top+2, FALSE, NULL, NULL);
         else
-            do_stat_label (_("Never Expires"), table, 1, top+2, FALSE, NULL, NULL);
+            do_stat_label (_("Never"), table, 1, top+2, FALSE, NULL, NULL);
    } else {
         date_edit = gnome_date_edit_new (subkey->expires, FALSE, FALSE);
         gtk_widget_show (date_edit);
@@ -224,7 +302,7 @@ do_stats (SeahorseWidget *swidget, GtkTable *table, guint top, guint index, gpgm
             g_signal_connect_after (GNOME_DATE_EDIT (date_edit), "date_changed",
                 G_CALLBACK (subkey_expires_date_changed), swidget);
         
-        widget = gtk_check_button_new_with_mnemonic (_("Never E_xpires"));
+        widget = gtk_check_button_new_with_mnemonic (_("_Never"));
         gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), !subkey->expires);
         gtk_tooltips_set_tip (tips, widget, _("If key never expires"), NULL);
         gtk_widget_show (widget);
@@ -240,34 +318,8 @@ do_stats (SeahorseWidget *swidget, GtkTable *table, guint top, guint index, gpgm
             g_signal_connect_after (GTK_TOGGLE_BUTTON (widget), "toggled",
                 G_CALLBACK (never_expires_subkey_toggled), swidget);
     }
-}
-
-static void
-do_key_info (SeahorseWidget *swidget)
-{
-    SeahorseKey *skey;
-    GtkWidget *w;
-    gchar *t, *x;
-
-    skey = SEAHORSE_KEY_WIDGET (swidget)->skey;
-    
-    w = glade_xml_get_widget (swidget->xml, "fingerprint");
-    t = seahorse_key_get_fingerprint (skey); 
-    gtk_label_set_text (GTK_LABEL (w), t);
-    g_free (t);
-    
-    w = glade_xml_get_widget (swidget->xml, "userid");
-    t = seahorse_key_get_userid (skey, 0); 
-    gtk_label_set_text (GTK_LABEL (w), t);
-    
-    x = g_strdup_printf (_("%s Properties"), t);
-    g_free (t);
-    
-    w = glade_xml_get_widget (swidget->xml, swidget->name);
-    gtk_window_set_title (GTK_WINDOW (w), x);
-    g_free (x);
-}
     
+}    
 
 static void
 passphrase_clicked (GtkWidget *widget, SeahorseWidget *swidget)
@@ -547,7 +599,7 @@ uid_primary_clicked (GtkWidget *widget, SeahorseWidget *swidget)
         err = seahorse_key_op_primary_uid (skey, index);
         
         if (!GPG_IS_OK (err)) 
-            seahorse_util_handle_error (err, _("Couldn't change primary user ID"));
+            seahorse_util_handle_gpgme (err, _("Couldn't change primary user ID"));
     }
 }
 
@@ -658,14 +710,14 @@ key_changed (SeahorseKey *skey, SeahorseKeyChange change, SeahorseWidget *swidge
 	switch (change) {
         case SKEY_CHANGE_ALL:
             do_uid_list (swidget);
-            do_key_info (swidget);
+            key_property_labels (swidget);
             do_signatures (swidget);
             do_signature_list (swidget);
             do_subkeys (swidget);
             break;
 		case SKEY_CHANGE_UIDS:
 			do_uid_list (swidget);
-            do_key_info (swidget);
+            key_property_labels (swidget);
 			break;
         case SKEY_CHANGE_SIGNERS:
             do_signatures (swidget);
@@ -695,20 +747,30 @@ properties_destroyed (GtkObject *object, SeahorseWidget *swidget)
                                           key_destroyed, swidget);
 }
 
+
+
+
 void
 seahorse_key_properties_new (SeahorseContext *sctx, SeahorseKey *skey)
 {
     SeahorseKeySource *sksrc;
 	SeahorseWidget *swidget;
 	GtkWidget *widget;
+    gboolean remote;
+    
+    remote = seahorse_key_get_loaded_info (skey) == SKEY_INFO_REMOTE;
     
     /* Reload the key to make sure to get all the props */
     sksrc = seahorse_key_get_source (skey);
     g_return_if_fail (sksrc != NULL);
         
-    skey = seahorse_key_source_get_key (sksrc, 
-                seahorse_key_get_id (skey->key), SKEY_INFO_COMPLETE);
-    g_return_if_fail (skey != NULL);                
+    /* Don't trigger the import of remote keys if possible */
+    if (!remote) {
+        /* This causes the key source to get any specific info about the key */
+        seahorse_key_source_refresh_sync (sksrc, seahorse_key_get_id (skey->key));
+        skey = seahorse_key_source_get_key (sksrc, seahorse_key_get_id (skey->key));
+        g_return_if_fail (skey != NULL);                
+    }
 	
 	swidget = seahorse_key_widget_new ("key-properties", sctx, skey);
 	g_return_if_fail (swidget != NULL);
@@ -718,11 +780,7 @@ seahorse_key_properties_new (SeahorseContext *sctx, SeahorseKey *skey)
     g_signal_connect_after (skey, "changed", G_CALLBACK (key_changed), swidget);
     g_signal_connect_after (skey, "destroy", G_CALLBACK (key_destroyed), swidget);
 	
-	do_stats (swidget, GTK_TABLE (glade_xml_get_widget (swidget->xml, "primary_table")), 3, 0,
-		skey->key->subkeys);
-     
-    /* Main key info */
-    do_key_info (swidget);
+	key_property_labels (swidget);
      
     /* Change password button */
     if(SEAHORSE_IS_KEY_PAIR (skey)) {
@@ -733,43 +791,56 @@ seahorse_key_properties_new (SeahorseContext *sctx, SeahorseKey *skey)
     }
 
 	do_subkeys (swidget);
-    
-    /* Uid page */
-    widget = glade_xml_get_widget (swidget->xml, "uid-list");
-    g_signal_connect (gtk_tree_view_get_selection (GTK_TREE_VIEW (widget)), "changed",
-                G_CALLBACK (uid_sel_changed), swidget);
-    glade_xml_signal_connect_data (swidget->xml, "uid_add_clicked",
-                G_CALLBACK (uid_add_clicked), swidget);
-    glade_xml_signal_connect_data (swidget->xml, "uid_primary_clicked",
-                G_CALLBACK (uid_primary_clicked), swidget);
-    glade_xml_signal_connect_data (swidget->xml, "uid_sign_clicked",
-                G_CALLBACK (uid_sign_clicked), swidget);
-    glade_xml_signal_connect_data (swidget->xml, "uid_delete_clicked",
-                G_CALLBACK (uid_delete_clicked), swidget);
     do_uid_list (swidget);
 
-    
-    /* Signature setup */
-    do_signatures (swidget);
-    do_signature_list (swidget);
-    glade_xml_signal_connect_data (swidget->xml, "sigs_changed",
-          G_CALLBACK (signature_sel_changed), swidget);        
+    /* Disable stuff not appropriate for remote */
+    if (remote) {
+        gtk_widget_set_sensitive (glade_xml_get_widget (swidget->xml, "trust"), FALSE);
+        gtk_widget_set_sensitive (glade_xml_get_widget (swidget->xml, "disabled"), FALSE);
 
-	/* disable trust options */
-	if (SEAHORSE_IS_KEY_PAIR (skey))
-		widget = glade_xml_get_widget (swidget->xml, "unknown");
-	else
-		widget = glade_xml_get_widget (swidget->xml, "ultimate");
-	gtk_widget_set_sensitive (widget, FALSE);
-	
-	gtk_option_menu_set_history (GTK_OPTION_MENU (glade_xml_get_widget (swidget->xml, "trust")),
-		seahorse_key_get_trust (skey) - 1);
+        /* Remove the signatures page (3rd page) */ 
+        widget = glade_xml_get_widget (swidget->xml, NOTEBOOK);
+        gtk_notebook_remove_page (GTK_NOTEBOOK (widget), 2);
+        
+    /* Stuff valid for local keys */
+    } else {     
+
+        /* Uid page */
+        widget = glade_xml_get_widget (swidget->xml, "uid-list");
+        g_signal_connect (gtk_tree_view_get_selection (GTK_TREE_VIEW (widget)), "changed",
+                    G_CALLBACK (uid_sel_changed), swidget);
+        glade_xml_signal_connect_data (swidget->xml, "uid_add_clicked",
+                    G_CALLBACK (uid_add_clicked), swidget);
+        glade_xml_signal_connect_data (swidget->xml, "uid_primary_clicked",
+                    G_CALLBACK (uid_primary_clicked), swidget);
+        glade_xml_signal_connect_data (swidget->xml, "uid_sign_clicked",
+                    G_CALLBACK (uid_sign_clicked), swidget);
+        glade_xml_signal_connect_data (swidget->xml, "uid_delete_clicked",
+                    G_CALLBACK (uid_delete_clicked), swidget);
+                    
+        /* Signature setup */
+        do_signatures (swidget);
+        do_signature_list (swidget);
+        glade_xml_signal_connect_data (swidget->xml, "sigs_changed",
+                                       G_CALLBACK (signature_sel_changed), swidget);        
+
+        /* disable trust options */
+        if (SEAHORSE_IS_KEY_PAIR (skey))
+            widget = glade_xml_get_widget (swidget->xml, "unknown");
+        else
+            widget = glade_xml_get_widget (swidget->xml, "ultimate");
+        gtk_widget_set_sensitive (widget, FALSE);
+        
+    	gtk_option_menu_set_history (GTK_OPTION_MENU (glade_xml_get_widget (swidget->xml, "trust")),
+	                                 seahorse_key_get_trust (skey) - 1);
+        glade_xml_signal_connect_data (swidget->xml, "trust_changed",
+                                       G_CALLBACK (trust_changed), swidget);
+        glade_xml_signal_connect_data (swidget->xml, "disabled_toggled",
+                                       G_CALLBACK (disabled_toggled), swidget);
+    }
 	
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (glade_xml_get_widget (swidget->xml, "disabled")),
-		skey->key->disabled);
-	
-	glade_xml_signal_connect_data (swidget->xml, "trust_changed",
-		G_CALLBACK (trust_changed), swidget);
-	glade_xml_signal_connect_data (swidget->xml, "disabled_toggled",
-		G_CALLBACK (disabled_toggled), swidget);
+		                          skey->key->disabled);
+
+    seahorse_widget_show (swidget);    
 }
